app

package
v0.2.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Sep 19, 2021 License: GPL-3.0 Imports: 14 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

var DefaultConfigYaml []byte

Functions

func LoadOrCreateConfig

func LoadOrCreateConfig(forceDefaultConfig bool) (config.RuleSet, error)

LoadOrCreateConfig loads the config file if it exists and creates a default config file otherwise.

Types

type Editor

type Editor struct {
	// contains filtered or unexported fields
}

Editor is a terminal-based text editing program.

func NewEditor

func NewEditor(screen tcell.Screen, path string, configRuleSet config.RuleSet) *Editor

NewEditor instantiates a new editor that uses the provided screen.

func (*Editor) RunEventLoop

func (e *Editor) RunEventLoop()

RunEventLoop processes events and draws to the screen, blocking until the user exits the program.

Jump to